using Microsoft.AspNetCore.Mvc;
using iFileProxy.Services;
using iFileProxy.Models;

namespace iFileProxy.Controllers
{
    [Route("[controller]")]
    [ApiController]
    public class GithubProxyController : ControllerBase
    {
        private readonly ILogger<GithubProxyController> _logger;
        private readonly GithubProxyService _proxyService;

        public GithubProxyController(
            ILogger<GithubProxyController> logger,
            GithubProxyService proxyService)
        {
            _logger = logger;
            _proxyService = proxyService;
        }

        [HttpGet("{**url}")]
        public async Task<IActionResult> ProxyDownload(string url)
        {
            try
            {
                // 1. URL 格式化
                url = url.StartsWith("http") ? url : $"https://{url}";

                // 2. 验证 URL
                var (isValid, author, repo) = _proxyService.ValidateUrl(url);
                if (!isValid || author == null || repo == null)
                {
                    return BadRequest(new CommonRsp 
                    { 
                        Retcode = 1,
                        Message = "Invalid GitHub URL" 
                    });
                }

                // 3. 检查黑名单
                if (_proxyService.IsBlocked(author, repo))
                {
                    return Forbid();
                }

                // 4. 处理URL（CDN或Raw）
                url = _proxyService.ProcessUrl(url);

                // 5. 代理请求
                var response = await _proxyService.ProxyRequestAsync(url, Request.Headers);
                if (!response.Success)
                {
                    return StatusCode(response.StatusCode, new CommonRsp
                    {
                        Retcode = 1,
                        Message = response.Message
                    });
                }

                // 6. 设置响应头
                if (response.Headers != null)
                {
                    foreach (var header in response.Headers)
                    {
                        Response.Headers[header.Key] = header.Value;
                    }
                }

                // 7. 返回流式响应
                return new FileStreamResult(
                    response.Stream!,
                    response.ContentType ?? "application/octet-stream"
                );
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Proxy download failed");
                return StatusCode(500, new CommonRsp 
                { 
                    Retcode = 1,
                    Message = "Internal server error" 
                });
            }
        }
    }
} 