﻿using iFileProxy.Config;
using Serilog;
using System.Data;
using iFileProxy.Helpers;
using MySql.Data.MySqlClient;
using iFileProxy.Models;
using Newtonsoft.Json;
using System.Text;
using Serilog.Events;

namespace iFileProxy.Services
{
    public class DatabaseGateService
    {
        Database _db;
        AppConfig _appConfig;
        private readonly Serilog.ILogger _logger = Log.Logger.ForContext<DatabaseGateService>();

        Dictionary<string, DB> _dbDictionary = [];

        private const string CREATE_TASKS_TABLE_SQL = """
            CREATE TABLE IF NOT EXISTS `t_tasks_info` (
                `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '自增键',
                `tid` varchar(48) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL COMMENT '任务id',
                `file_name` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL,
                `client_ip` varchar(15) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL DEFAULT NULL COMMENT '客户端IP',
                `add_time` datetime NOT NULL COMMENT '任务在何时被添加',
                `update_time` datetime NOT NULL COMMENT '任务状态更新时间',
                `status` int(11) NULL DEFAULT NULL COMMENT '任务状态',
                `url` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL DEFAULT NULL COMMENT '文件url',
                `size` int(11) NULL DEFAULT NULL COMMENT '文件大小',
                `hash` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL DEFAULT NULL COMMENT '文件hash',
                PRIMARY KEY (`id`) USING BTREE
            ) ENGINE = InnoDB AUTO_INCREMENT = 17 CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci ROW_FORMAT = Dynamic;
        """;

        private const string CREATE_USERS_TABLE_SQL = """
            CREATE TABLE IF NOT EXISTS `t_users` (
                `user_id` varchar(36) NOT NULL,
                `username` varchar(50) NOT NULL,
                `nickname` varchar(50) NOT NULL,
                `email` varchar(100) NOT NULL,
                `password_hash` varchar(255) NOT NULL,
                `mask` int NOT NULL DEFAULT 0,
                `state` int NOT NULL DEFAULT 0,
                `create_time` datetime NOT NULL,
                `last_login_time` datetime NULL,
                `last_login_ip` varchar(45) NULL,
                PRIMARY KEY (`user_id`),
                UNIQUE KEY `username` (`username`),
                UNIQUE KEY `email` (`email`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
        """;

        private const string CREATE_USER_EVENTS_TABLE_SQL = """
            CREATE TABLE IF NOT EXISTS `t_user_events` (
                `event_id` varchar(36) NOT NULL,
                `user_id` varchar(36) NOT NULL,
                `event_type` int NOT NULL,
                `event_time` datetime NOT NULL,
                `event_ip` varchar(45) NULL,
                `event_detail` text NULL,
                PRIMARY KEY (`event_id`),
                KEY `user_id` (`user_id`),
                CONSTRAINT `fk_user_events_user` FOREIGN KEY (`user_id`) REFERENCES `t_users` (`user_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
        """;

        private const string CREATE_SYSTEM_LOGS_TABLE_SQL = """
            CREATE TABLE IF NOT EXISTS `t_system_logs` (
                `log_id` varchar(36) NOT NULL,
                `level` varchar(20) NOT NULL,
                `message` text NOT NULL,
                `exception` text,
                `properties` json,
                `timestamp` datetime NOT NULL,
                PRIMARY KEY (`log_id`),
                KEY `idx_timestamp` (`timestamp`),
                KEY `idx_level` (`level`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
        """;

        public DatabaseGateService(AppConfig appConfig)
        {
            _logger.Information("Initializing DatabaseGateService...");
            _db = appConfig.Database;
            _appConfig = appConfig;
            try
            {
                _logger.Information("Done.");
            }
            catch (Exception e)
            {
                _logger.Fatal($"程序异常: {e.Message}");
            }
            LoadDbDict();
        }

        /// <summary>
        /// 加载数据库描述字典
        /// </summary>
        public void LoadDbDict()
        {
            foreach (DB item in _db.Databases)
            {
                _dbDictionary.Add(item.Description, item);
                _logger.Debug($"Db Config: {item.Description} <=> {item.DatabaseName} Loaded.");
            }
        }
        /// <summary>
        /// 获取一个指定数据库的连接
        /// </summary>
        /// <param name="db_desc">数据库描述字段 对应AppConfig的description字段</param>
        /// <returns></returns>
        /// <exception cref="Exception">若某些不允许为空的字段出现空值 则抛出此异常</exception>
        /// 
        public MySqlConnection GetAndOpenDBConn(string db_desc)
        {
            if (!_dbDictionary.TryGetValue(db_desc, out DB Db))
            {
                throw new Exception($"未找到与 {db_desc} 相匹配的数据库配置");
            }

            var db_user = Db.User ?? _db.Common.User;
            var db_password = Db.Password ?? _db.Common.Password;
            var db_host = Db.Host ?? _db.Common.Host;
            var db_port = Db.Port ?? _db.Common.Port;

            if (db_user == null || db_password == null || db_host == null || db_port == null)
                throw new NoNullAllowedException("数据库配置获取失败,不允许为空的字段出现空值");

            var builder = new MySqlConnectionStringBuilder
            {
                Server = db_host,
                UserID = db_user,
                Database = Db.DatabaseName,
                Port = (uint)db_port,
                Password = db_password,
                Pooling = true,
                MinimumPoolSize = 1,
                MaximumPoolSize = AppConfig.GetCurrConfig().Database.MaxConnectionPoolSize,
                ConnectionLifeTime = 300  // 连接最大生命周期(秒)
            };

            try
            {
                var conn = new MySqlConnection(builder.ConnectionString);
                conn.Open();
                return conn;
            }
            catch (Exception ex)
            {
                _logger.Fatal($"获取Mysql连接时出现异常:{ex.Message}");
                throw;
            }
        }

        public bool TestDbConfig()
        {
            foreach (var db in _dbDictionary)
            {
                _logger.Information($"[程序启动前配置验证] 正在测试数据库配置: {db.Key} ...");
                MySqlConnection dbConn = new();
                try
                {
                    dbConn = GetAndOpenDBConn(db.Key);
                    _logger.Information($"succ.");
                }
                catch (Exception)
                {
                    _logger.Fatal($"=========== 数据库: {db.Key} 测试失败! ===========");
                    return false;
                }
                finally
                {
                    dbConn.Close();
                }
            }
            return true;
        }

        /// <summary>
        /// 获取一个json格式的数据表
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static string QueryTableData(MySqlCommand sqlCmd)
        {
            var dataTable = new DataTable();
            using (var adapter = new MySqlDataAdapter(sqlCmd))
            {
                adapter.Fill(dataTable);
            }
            return JsonConvert.SerializeObject(dataTable);
        }

        /// <summary>
        /// 获取一个json格式的数据表
        /// </summary>
        /// <param name="sql"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public  string QueryTableData(string sql,string dbConfName)
        {
            DataTable dataTable = new();
            using (MySqlDataAdapter adapter = new(new MySqlCommand(sql,GetAndOpenDBConn(dbConfName))))
                adapter.Fill(dataTable);
            return JsonConvert.SerializeObject(dataTable);
        }

        /// <summary>
        /// 内部查询数据专用 当此方法暴露给C端可能造成sql注入等安全问题
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <param name="dbConfName">配置文件中的Description字段</param>
        /// <returns>影响的行数</returns>
        public int Query(string sql, string dbConfName)
        { 
            using MySqlCommand sqlCmd = new (sql, GetAndOpenDBConn(dbConfName));
            int n = sqlCmd.ExecuteNonQuery();
            _logger.Debug($"查询完成, 受影响的行数: {n}");
            return n;
        }

        /// <summary>
        /// 内部查询数据专用 当此方法暴露给C端可能造成sql注入等安全问题
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <param name="dbConfName">配置文件中的Description字段</param>
        /// <returns>影响的行数</returns>
        public int Query(string sql, MySqlConnection connection)
        {
            using MySqlCommand sqlCmd = new(sql, connection);
            int n = sqlCmd.ExecuteNonQuery();
            _logger.Debug($"查询完成, 受影响的行数: {n}");
            return n;
        }

        public List<TaskInfo> CheckCacheDependencies(string taskId,string ipAddr)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE `status` = @status AND `tag` = @tag AND `client_ip` <> @ip_addr";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tag", $"REDIRECT:{taskId}");
                sqlCmd.Parameters.AddWithValue("@status", TaskState.Cached);
                sqlCmd.Parameters.AddWithValue("@ip_addr", ipAddr);

                return JsonConvert.DeserializeObject<List<TaskInfo>>( QueryTableData(sqlCmd));
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务列表: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }

        public string GetTaskListByStateAndIp(string ipAddr, TaskState status)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE client_ip = @ip_addr AND `status` = @status";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@ip_addr", ipAddr);
                sqlCmd.Parameters.AddWithValue ("@status", status);
                return QueryTableData(sqlCmd);
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务列表: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }


        public string GetTaskListByIP(string ipAddr)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE client_ip = @ip_addr";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new (sql,conn);
                sqlCmd.Parameters.AddWithValue("@ip_addr", ipAddr);
                return QueryTableData(sqlCmd);
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务列表: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }
        public string GetTaskInfoByTid(string tid)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE `tid` = @tid";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tid", tid);
                return QueryTableData(sqlCmd);
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务信息: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }

        public TaskInfo? QueryTaskInfo(string fileName, string url, long size, TaskState status)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE url = @url AND size = @size AND `status` = @status AND file_name = @fileName";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try {
                MySqlCommand sqlCmd = new MySqlCommand(sql, conn);
                sqlCmd.Parameters.AddWithValue("@url", url);
                sqlCmd.Parameters.AddWithValue("@size", size);
                sqlCmd.Parameters.AddWithValue("@status", status);
                sqlCmd.Parameters.AddWithValue("@fileName", fileName);
                string result = QueryTableData(sqlCmd);
                List<TaskInfo>? r = JsonConvert.DeserializeObject<List<TaskInfo>>(result);
                if (r != null)
                {
                    return r[0];
                }
                else
                    return null;
            }
            catch (Exception e){
                return null;
            }
            finally { 
                conn.Close();
            }

        }

        public bool InsertTaskData(TaskInfo taskInfo)
        {
            _logger.Debug(System.Text.Json.JsonSerializer.Serialize(taskInfo));
            string sql = "INSERT INTO `t_tasks_info` (`tid`, `file_name`, `client_ip`, `add_time`, `update_time`, `status`, `url`, `size`, `hash`, `tag`) " +
                "VALUES (@tid, @file_name, @client_ip, @add_time, @update_time, @status, @url, @size, @hash, @tag)";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");

            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tid", taskInfo.TaskId);
                sqlCmd.Parameters.AddWithValue("@file_name", taskInfo.FileName);
                sqlCmd.Parameters.AddWithValue("@client_ip", taskInfo.ClientIp);
                sqlCmd.Parameters.AddWithValue("@add_time", taskInfo.AddTime.ToString("yyyy-MM-dd HH:mm:ss"));
                sqlCmd.Parameters.AddWithValue("@update_time", taskInfo.UpdateTime.ToString("yyyy-MM-dd HH:mm:ss"));
                sqlCmd.Parameters.AddWithValue("@status", taskInfo.Status);
                sqlCmd.Parameters.AddWithValue("@url", taskInfo.Url);
                sqlCmd.Parameters.AddWithValue("@size", taskInfo.Size);
                sqlCmd.Parameters.AddWithValue("@hash", taskInfo.Hash);
                sqlCmd.Parameters.AddWithValue("@tag", taskInfo.Tag);

                sqlCmd.ExecuteNonQuery();
            }
            catch (Exception)
            {
                _logger.Fatal($"插入数据时出现问题");
                throw;
            }
            finally
            {
                conn.Close();
            }
            return true;
        }
        public bool UpdateFieldsData(string fieldsName, string taskUUID,object val)
        {
            string sql = $"UPDATE t_tasks_info set `{fieldsName}` = @Data WHERE `tid` = @tid";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@Data",val);
                sqlCmd.Parameters.AddWithValue("@tid",taskUUID);
                if (sqlCmd.ExecuteNonQuery() == 1)
                {
                    return true;
                }
                else
                return false;
            }
            catch (Exception)
            {
                _logger.Fatal($"Task Data update error.");
                throw;
            }
            finally
            {
                conn.Close();
            }

        }

        public bool UpdateTaskStatus(TaskInfo taskInfo, MySqlConnection? connection = null)
        {
            string sql = @"UPDATE t_tasks_info set `status` = @status , update_time = Now() WHERE `tid` = @tid";
            MySqlConnection conn = connection ?? GetAndOpenDBConn("iFileProxy_Db");
            
            try
            {
                using MySqlCommand sqlCmd = new (sql, conn);
                sqlCmd.Parameters.AddWithValue("@status", taskInfo.Status);
                sqlCmd.Parameters.AddWithValue("@tid", taskInfo.TaskId);
                if (sqlCmd.ExecuteNonQuery() >= 1)
                { 
                    _logger.Information($"Task: {taskInfo.TaskId} Status Change to {taskInfo.Status}"); 
                    return true;
                }
                else
                    _logger.Warning($"Task: {taskInfo.TaskId} Status Change Failed.");
                return false;
            }
            catch (Exception)
            {
                _logger.Fatal($"Task status update error.");
                throw;
            }
            finally
            { 
                conn.Close ();
            }
        }

        public bool UpdateTaskHash(TaskInfo taskInfo)
        {
            return UpdateFieldsData("hash", taskInfo.TaskId, MasterHelper.GetFileHash(Path.Combine(_appConfig.DownloadOptions.SavePath, taskInfo.FileName), FileHashAlgorithm.MD5));
        }

        /// <summary>
        /// 删除指定IP的任务信息
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        public int DeleteTaskInfoByIpAddr(string ipAddr)
        { 
            try
            {
                string sql = $"DELETE FROM `t_tasks_info` WHERE `client_ip` = '{ipAddr}'";
                return Query(sql,DbConfigName.iFileProxy);
            }
            catch (Exception)
            {
                return -1;
            }
        }

        public void TryInitialDB()
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                // 创建表
                using (var cmd = new MySqlCommand(CREATE_TASKS_TABLE_SQL, conn))
                    cmd.ExecuteNonQuery();
                using (var cmd = new MySqlCommand(CREATE_USERS_TABLE_SQL, conn))
                    cmd.ExecuteNonQuery();
                using (var cmd = new MySqlCommand(CREATE_USER_EVENTS_TABLE_SQL, conn))
                    cmd.ExecuteNonQuery();

                _logger.Information("数据库表结构初始化完成");
            }
            catch (Exception ex)
            {
                _logger.Error($"数据库初始化失败: {ex.Message}");
                throw;
            }
        }
        public T ExecuteScalar<T>(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }

            return (T)cmd.ExecuteScalar();
        }

        public List<T> ExecuteQuery<T>(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return JsonConvert.DeserializeObject<List<T>>(QueryTableData(cmd));
        }

        public int ExecuteNonQuery(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return cmd.ExecuteNonQuery();
        }

        /// <summary>
        /// 获取分页的任务列表
        /// </summary>
        /// <param name="page">页码，从1开始</param>
        /// <param name="pageSize">每页数量</param>
        /// <param name="status">可选的任务状态过滤</param>
        /// <returns>包含分页信息的任务列表</returns>
        public PagedResult<TaskInfo> GetPagedTaskList(int page, int pageSize, TaskState? status = null)
        {
            // 构建基础SQL
            var sql = new StringBuilder("SELECT * FROM t_tasks_info");
            var parameters = new Dictionary<string, object>();

            // 添加状态过滤
            if (status.HasValue)
            {
                sql.Append(" AND status = @status");
                parameters.Add("@status", (int)status.Value);
            }

            // 添加分页
            sql.Append(" ORDER BY add_time DESC LIMIT @offset, @limit");
            parameters.Add("@offset", (page - 1) * pageSize);
            parameters.Add("@limit", pageSize);

            // 获取总记录数
            var countSql = "SELECT COUNT(*) FROM t_tasks_info" +
                (status.HasValue ? " AND status = @status" : "");
            var totalCount = ExecuteScalar<long>(countSql, parameters);

            // 获取分页数据
            var tasks = ExecuteQuery<TaskInfo>(sql.ToString(), parameters);

            // 返回结果
            return new PagedResult<TaskInfo>
            {
                Total = totalCount,
                PageSize = pageSize,
                CurrentPage = page,
                TotalPages = (int)Math.Ceiling(totalCount / (double)pageSize),
                Data = tasks
            };
        }

        /// <summary>
        /// 根据任务ID删除任务
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <returns>是否删除成功</returns>
        public bool DeleteTaskByTid(string taskId)
        {
            try
            {
                var sql = "DELETE FROM t_tasks_info WHERE tid = @taskId";
                var parameters = new Dictionary<string, object>
                {
                    { "@taskId", taskId }
                };

                var affectedRows = ExecuteNonQuery(sql, parameters);
                return affectedRows > 0;
            }
            catch (Exception ex)
            {
                _logger.Error($"删除任务失败 [TaskId: {taskId}]: {ex.Message}");
                return false;
            }
        }

        /// <summary>
        /// 获取任务的详细信息
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <returns>任务详细信息</returns>
        public TaskInfo GetTaskDetail(string taskId)
        {
            try
            {
                var sql = """
                    SELECT * FROM t_tasks_info WHERE tid = @taskId
                    """;

                var parameters = new Dictionary<string, object>
                {
                    { "@taskId", taskId }
                };

                var taskInfo = ExecuteQuery<TaskInfo>(sql, parameters).FirstOrDefault();
                return taskInfo;
            }
            catch (Exception ex)
            {
                _logger.Error($"获取任务详情失败 [TaskId: {taskId}]: {ex.Message}");
                throw;
            }
        }

        /// <summary>
        /// 获取各状态任务的数量统计
        /// </summary>
        /// <returns>以任务状态为键，数量为值的字典</returns>
        public Dictionary<TaskState, int> GetTaskStatusOverview()
        {
            try
            {
                var sql = """
                    SELECT 
                        status,
                        COUNT(*) as count
                    FROM t_tasks_info 
                    GROUP BY status
                    """;

                var result = new Dictionary<TaskState, int>();
                
                // 初始化所有状态的计数为0
                foreach (TaskState state in Enum.GetValues(typeof(TaskState)))
                {
                    result[state] = 0;
                }

                // 获取数据库中的实际计数
                using var conn = GetAndOpenDBConn("iFileProxy_Db");
                using var cmd = new MySqlCommand(sql, conn);
                using var reader = cmd.ExecuteReader();
                
                while (reader.Read())
                {
                    var status = (TaskState)reader.GetInt32("status");
                    var count = reader.GetInt32("count");
                    result[status] = count;
                }

                return result;
            }
            catch (Exception ex)
            {
                _logger.Error($"获取任务状态统计失败: {ex.Message}");
                throw;
            }
        }

        public async Task<User?> GetUserByIdAsync(string userId)
        {
            var sql = "SELECT * FROM t_users WHERE user_id = @userId";
            var parameters = new Dictionary<string, object>
            {
                { "@userId", userId }
            };

            var users = await ExecuteQueryAsync<User>(sql, parameters);
            return users.FirstOrDefault();
        }

        public async Task<bool> UserExistsAsync(string username, string email)
        {
            var sql = "SELECT COUNT(*) FROM t_users WHERE username = @username OR email = @email";
            var parameters = new Dictionary<string, object>
            {
                { "@username", username },
                { "@email", email }
            };
            var count = await ExecuteScalarAsync<long>(sql, parameters);
            return count > 0;
        }

        public async Task<bool> CreateUserAsync(User user)
        {
            var sql = @"INSERT INTO t_users (user_id, username, password_hash, mask, state, create_time, last_login_time, last_login_ip, nickname, email) 
                       VALUES (@userId, @username, @passwordHash, @mask, @state, @createTime, @lastLoginTime, @lastLoginIp, @nickname, @email)";
            
            var parameters = new Dictionary<string, object>
            {
                { "@userId", user.UserId },
                { "@username", user.Username },
                { "@passwordHash", user.PasswordHash },
                { "@mask", (int)user.Mask },
                { "@state", (int)user.State },
                { "@createTime", user.CreateTime },
                { "@lastLoginTime", user.LastLoginTime },
                { "@lastLoginIp", user.LastLoginIP },
                { "@nickname", user.Nickname },
                { "@email", user.Email }
            };

            var result = await ExecuteNonQueryAsync(sql, parameters);
            return result > 0;
        }

        public async Task<bool> CreateUserEventAsync(UserEvent userEvent)
        {
            var sql = @"INSERT INTO t_user_events (event_id, user_id, event_type, event_time, event_ip, event_detail) 
                       VALUES (@eventId, @userId, @eventType, @eventTime, @eventIp, @eventDetail)";
            
            var parameters = new Dictionary<string, object>
            {
                { "@eventId", userEvent.EventId },
                { "@userId", userEvent.UserId },
                { "@eventType", (int)userEvent.EventType },
                { "@eventTime", userEvent.EventTime },
                { "@eventIp", userEvent.EventIP },
                { "@eventDetail", userEvent.EventDetail }
            };

            var result = await ExecuteNonQueryAsync(sql, parameters);
            return result > 0;
        }

        public async Task<User?> GetUserByUsernameAsync(string username)
        {
            var sql = "SELECT * FROM t_users WHERE username = @username";
            var parameters = new Dictionary<string, object>
            {
                { "@username", username }
            };

            var users = await ExecuteQueryAsync<User>(sql, parameters);
            return users.FirstOrDefault();
        }

        public async Task<bool> UpdateUserAsync(User user)
        {
            var sql = @"UPDATE t_users SET 
                       username = @username,
                       password_hash = @passwordHash,
                       mask = @mask,
                       state = @state,
                       last_login_time = @lastLoginTime,
                       last_login_ip = @lastLoginIp,
                       nickname = @nickname
                       WHERE user_id = @userId";
            
            var parameters = new Dictionary<string, object>
            {
                { "@userId", user.UserId },
                { "@username", user.Username },
                { "@passwordHash", user.PasswordHash },
                { "@mask", (int)user.Mask },
                { "@state", (int)user.State },
                { "@lastLoginTime", user.LastLoginTime },
                { "@lastLoginIp", user.LastLoginIP },
                { "@nickname", user.Nickname }

            };

            var result = await ExecuteNonQueryAsync(sql, parameters);
            return result > 0;
        }

        private async Task<T> ExecuteScalarAsync<T>(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return (T)await cmd.ExecuteScalarAsync();
        }

        private async Task<List<T>> ExecuteQueryAsync<T>(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }

            // 使用 DataTable 来避免 DataReader 冲突
            var dataTable = new DataTable();
            using (var adapter = new MySqlDataAdapter(cmd))
            {
                await Task.Run(() => adapter.Fill(dataTable));
            }

            return JsonConvert.DeserializeObject<List<T>>(
                JsonConvert.SerializeObject(dataTable)
            );
        }

        private async Task<int> ExecuteNonQueryAsync(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return await cmd.ExecuteNonQueryAsync();
        }

        public async Task<long> GetUserCountAsync()
        {
            var sql = "SELECT COUNT(*) FROM t_users";
            var parameters = new Dictionary<string, object>();
            return await ExecuteScalarAsync<long>(sql, parameters);
        }

        public async Task<PagedResult<User>> GetPagedUserListAsync(int page, int pageSize, UserMask? mask = null)
        {
            // 构建基础SQL
            var sql = new StringBuilder("SELECT * FROM t_users");
            var parameters = new Dictionary<string, object>();

            // 添加权限过滤
            if (mask.HasValue)
            {
                sql.Append(" WHERE mask = @mask");
                parameters.Add("@mask", (int)mask.Value);
            }

            // 添加分页
            sql.Append(" ORDER BY create_time DESC LIMIT @offset, @limit");
            parameters.Add("@offset", (page - 1) * pageSize);
            parameters.Add("@limit", pageSize);

            // 获取总记录数
            var countSql = "SELECT COUNT(*) FROM t_users" +
                (mask.HasValue ? " WHERE mask = @mask" : "");
            var totalCount = await ExecuteScalarAsync<long>(countSql, parameters);

            // 获取分页数据
            var users = await ExecuteQueryAsync<User>(sql.ToString(), parameters);

            return new PagedResult<User>
            {
                Total = totalCount,
                PageSize = pageSize,
                CurrentPage = page,
                TotalPages = (int)Math.Ceiling(totalCount / (double)pageSize),
                Data = users
            };
        }

        public async Task<PagedResult<UserEventDetail>> GetPagedUserEventsAsync(
            int page, 
            int pageSize, 
            string? userId = null, 
            string? keyword = null)
        {
            // 构建基础SQL
            var sql = new StringBuilder(@"
                SELECT e.*, u.username, u.nickname 
                FROM t_user_events e
                LEFT JOIN t_users u ON e.user_id = u.user_id
                WHERE 1=1");
            
            var parameters = new Dictionary<string, object>();

            // 添加用户ID过滤
            if (!string.IsNullOrEmpty(userId))
            {
                sql.Append(" AND e.user_id = @userId");
                parameters.Add("@userId", userId);
            }

            // 添加关键词搜索
            if (!string.IsNullOrEmpty(keyword))
            {
                sql.Append(@" AND (
                    e.event_detail LIKE @keyword OR 
                    u.username LIKE @keyword OR 
                    u.nickname LIKE @keyword OR
                    e.event_ip LIKE @keyword
                )");
                parameters.Add("@keyword", $"%{keyword}%");
            }

            // 添加分页
            sql.Append(" ORDER BY e.event_time DESC LIMIT @offset, @limit");
            parameters.Add("@offset", (page - 1) * pageSize);
            parameters.Add("@limit", pageSize);

            // 获取总记录数
            var countSql = @"
                SELECT COUNT(*) 
                FROM t_user_events e 
                LEFT JOIN t_users u ON e.user_id = u.user_id
                WHERE 1=1" +
                (!string.IsNullOrEmpty(userId) ? " AND e.user_id = @userId" : "") +
                (!string.IsNullOrEmpty(keyword) ? @" AND (
                    e.event_detail LIKE @keyword OR 
                    u.username LIKE @keyword OR 
                    u.nickname LIKE @keyword OR
                    e.event_ip LIKE @keyword
                )" : "");

            var totalCount = await ExecuteScalarAsync<long>(countSql, parameters);

            // 获取分页数据
            var events = await ExecuteQueryAsync<UserEventDetail>(sql.ToString(), parameters);

            return new PagedResult<UserEventDetail>
            {
                Total = totalCount,
                PageSize = pageSize,
                CurrentPage = page,
                TotalPages = (int)Math.Ceiling(totalCount / (double)pageSize),
                Data = events
            };
        }

        public async Task<User?> GetUserByAccountAsync(string account)
        {
            var sql = "SELECT * FROM t_users WHERE username = @account OR email = @account";
            var parameters = new Dictionary<string, object>
            {
                { "@account", account }
            };

            var users = await ExecuteQueryAsync<User>(sql, parameters);
            return users.FirstOrDefault();
        }

        /// <summary>
        /// 获取数据库连接池状态
        /// </summary>
        /// <returns>连接池信息</returns>
        public async Task<ConnectionPoolInfo> GetConnectionPoolInfoAsync()
        {
            try
            {
                var appName = "iFileProxy";  // 程序名称
                var parameters = new Dictionary<string, object>
                {
                    { "@appName", $"%{appName}%" }
                };

                // 获取总连接数
                var totalSql = @"SELECT COUNT(*) 
                                FROM information_schema.processlist 
                                WHERE `USER` LIKE @appName OR  `DB` LIKE @appName";
                var totalConnections = await ExecuteScalarAsync<long>(totalSql, parameters);

                // 获取活跃连接数
                var activeSql = @"SELECT COUNT(*) 
                                 FROM information_schema.processlist 
                                 WHERE Command != 'Sleep' 
                                 AND ( `USER` LIKE @appName OR  `DB` LIKE @appName)";
                var activeConnections = await ExecuteScalarAsync<long>(activeSql, parameters);

                // 获取睡眠连接数
                var sleepingSql = @"SELECT COUNT(*) 
                                   FROM information_schema.processlist 
                                   WHERE Command = 'Sleep' 
                                   AND ( `USER` LIKE @appName OR  `DB` LIKE @appName)";
                var sleepingConnections = await ExecuteScalarAsync<long>(sleepingSql, parameters);

                // 获取连接详情
                var detailSql = @"SELECT id, user, host, db, command, time, state, info 
                                 FROM information_schema.processlist 
                                 WHERE `USER` LIKE @appName OR `DB` LIKE @appName";
                var connections = await ExecuteQueryAsync<ProcessListInfo>(detailSql, parameters);

                return new ConnectionPoolInfo
                {
                    TotalConnections = totalConnections,
                    ActiveConnections = activeConnections,
                    SleepingConnections = sleepingConnections,
                    MaxPoolSize = AppConfig.GetCurrConfig().Database.MaxConnectionPoolSize,  // 从配置中获取的最大连接数
                    Connections = connections
                };
            }
            catch (Exception ex)
            {
                _logger.Error($"获取连接池信息失败: {ex.Message}");
                throw;
            }
        }

        public async Task<bool> CreateSystemLogAsync(SystemLog log)
        {
            var sql = @"INSERT INTO t_system_logs 
                       (log_id, level, message, exception, properties, timestamp) 
                       VALUES 
                       (@logId, @level, @message, @exception, @properties, @timestamp)";
            
            var parameters = new Dictionary<string, object>
            {
                { "@logId", log.LogId },
                { "@level", log.Level },
                { "@message", log.Message },
                { "@exception", log.Exception },
                { "@properties", log.Properties },
                { "@timestamp", log.Timestamp }
            };

            var result = await ExecuteNonQueryAsync(sql, parameters);
            return result > 0;
        }

        public async Task<PagedResult<SystemLog>> GetPagedSystemLogsAsync(
            int page, 
            int pageSize, 
            LogEventLevel? level = null,
            string? keyword = null,
            DateTime? startTime = null,
            DateTime? endTime = null)
        {
            // 构建基础SQL
            var sql = new StringBuilder("SELECT * FROM t_system_logs WHERE 1=1");
            var parameters = new Dictionary<string, object>();

            // 添加日志级别过滤
            if (level.HasValue)
            {
                sql.Append(" AND level = @level");
                parameters.Add("@level", level.Value.ToString());
            }

            // 添加关键词搜索
            if (!string.IsNullOrEmpty(keyword))
            {
                sql.Append(" AND (message LIKE @keyword OR exception LIKE @keyword)");
                parameters.Add("@keyword", $"%{keyword}%");
            }

            // 添加时间范围过滤
            if (startTime.HasValue)
            {
                sql.Append(" AND timestamp >= @startTime");
                parameters.Add("@startTime", startTime.Value);
            }
            if (endTime.HasValue)
            {
                sql.Append(" AND timestamp <= @endTime");
                parameters.Add("@endTime", endTime.Value);
            }

            // 添加分页和排序
            sql.Append(" ORDER BY timestamp DESC LIMIT @offset, @limit");
            parameters.Add("@offset", (page - 1) * pageSize);
            parameters.Add("@limit", pageSize);

            // 获取总记录数
            var countSql = "SELECT COUNT(*) FROM t_system_logs WHERE 1=1" +
                (level.HasValue ? " AND level = @level" : "") +
                (!string.IsNullOrEmpty(keyword) ? " AND (message LIKE @keyword OR exception LIKE @keyword)" : "") +
                (startTime.HasValue ? " AND timestamp >= @startTime" : "") +
                (endTime.HasValue ? " AND timestamp <= @endTime" : "");

            var totalCount = await ExecuteScalarAsync<long>(countSql, parameters);
            var logs = await ExecuteQueryAsync<SystemLog>(sql.ToString(), parameters);

            return new PagedResult<SystemLog>
            {
                Total = totalCount,
                PageSize = pageSize,
                CurrentPage = page,
                TotalPages = (int)Math.Ceiling(totalCount / (double)pageSize),
                Data = logs
            };
        }
    }
}
