using System.Text.RegularExpressions;
using iFileProxy.Config;
using iFileProxy.Models;

namespace iFileProxy.Services
{
    public class GithubProxyService
    {
        private readonly ILogger<GithubProxyService> _logger;
        private readonly AppConfig _appConfig;
        private readonly HttpClient _httpClient;

        private static readonly Regex[] URL_PATTERNS = 
        {
            new(@"^(?:https?://)?github\.com/(?<author>.+?)/(?<repo>.+?)/(?:releases|archive)/.*$"),
            new(@"^(?:https?://)?github\.com/(?<author>.+?)/(?<repo>.+?)/(?:blob|raw)/.*$"),
            new(@"^(?:https?://)?raw\.(?:githubusercontent|github)\.com/(?<author>.+?)/(?<repo>.+?)/.+?/.+$"),
            new(@"^(?:https?://)?gist\.(?:githubusercontent|github)\.com/(?<author>.+?)/.+?/.+$")
        };

        public GithubProxyService(
            ILogger<GithubProxyService> logger,
            AppConfig appConfig,
            HttpClient httpClient)
        {
            _logger = logger;
            _appConfig = appConfig;
            _httpClient = httpClient;
        }

        public (bool isValid, string? author, string? repo) ValidateUrl(string url)
        {
            foreach (var pattern in URL_PATTERNS)
            {
                var match = pattern.Match(url);
                if (match.Success)
                {
                    return (true, match.Groups["author"].Value, match.Groups["repo"].Value);
                }
            }
            return (false, null, null);
        }

        public bool IsBlocked(string author, string repo)
        {
            return _appConfig.GithubProxyOptions.Blacklist.Contains($"{author}/{repo}") || 
                   _appConfig.GithubProxyOptions.Blacklist.Contains(author);
        }

        public string ProcessUrl(string url)
        {
            if (_appConfig.GithubProxyOptions.EnableJsDelivr && url.Contains("/blob/"))
            {
                return url.Replace("/blob/", "@")
                         .Replace("github.com", "cdn.jsdelivr.net/gh");
            }
            else if (url.Contains("/blob/"))
            {
                return url.Replace("/blob/", "/raw/");
            }
            return url;
        }

        public async Task<ProxyResponse> ProxyRequestAsync(
            string url, 
            IHeaderDictionary headers, 
            CancellationToken cancellationToken = default)
        {
            try
            {
                var request = new HttpRequestMessage(HttpMethod.Get, url);
                
                // 复制请求头
                foreach (var header in headers)
                {
                    if (!header.Key.Equals("Host", StringComparison.OrdinalIgnoreCase))
                    {
                        request.Headers.TryAddWithoutValidation(header.Key, header.Value.ToArray());
                    }
                }

                var response = await _httpClient.SendAsync(
                    request, 
                    HttpCompletionOption.ResponseHeadersRead,
                    cancellationToken);

                // 检查文件大小
                var contentLength = response.Content.Headers.ContentLength;
                if (contentLength.HasValue && contentLength.Value > _appConfig.GithubProxyOptions.SizeLimit)
                {
                    return new ProxyResponse
                    {
                        Success = false,
                        StatusCode = StatusCodes.Status413PayloadTooLarge,
                        Message = "File too large"
                    };
                }

                return new ProxyResponse
                {
                    Success = true,
                    StatusCode = (int)response.StatusCode,
                    Headers = response.Headers.ToDictionary(h => h.Key, h => h.Value.ToArray()),
                    Stream = await response.Content.ReadAsStreamAsync(cancellationToken),
                    ContentType = response.Content.Headers.ContentType?.ToString()
                };
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Proxy request failed");
                return new ProxyResponse
                {
                    Success = false,
                    StatusCode = StatusCodes.Status500InternalServerError,
                    Message = "Internal server error"
                };
            }
        }
    }

    public class ProxyResponse
    {
        public bool Success { get; set; }
        public int StatusCode { get; set; }
        public string? Message { get; set; }
        public Dictionary<string, string[]>? Headers { get; set; }
        public Stream? Stream { get; set; }
        public string? ContentType { get; set; }
    }
} 